@@ -745,7 +745,8 @@ class LinkModule(mp_module.MPModule):
                 res = mavutil.mavlink.enums["MAV_RESULT"][m.result].name
                 res = res[11:]
                 if (m.target_component not in [mavutil.mavlink.MAV_COMP_ID_MAVCAN] and
-                    m.command not in [mavutil.mavlink.MAV_CMD_GET_HOME_POSITION]):
+                    m.command not in [mavutil.mavlink.MAV_CMD_GET_HOME_POSITION,
+                                      mavutil.mavlink.MAV_CMD_DO_DIGICAM_CONTROL]):
                     self.mpstate.console.writeln("Got COMMAND_ACK: %s: %s" % (cmd, res))
             except Exception:
                 self.mpstate.console.writeln("Got MAVLink msg: %s" % m)
